import 'package:dabric/utils/delay.dart';
import 'package:dabric/utils/detect.dart';
import 'package:objd/basic/file.dart';
import 'package:objd/basic/for_list.dart';
import 'package:objd/basic/pack.dart' as Objd;
import 'package:objd/basic/widget.dart';
import 'package:objd/build/context.dart';
import 'package:objd/build/project.dart' as Objd;

import 'pack.dart';

abstract class Project extends Widget implements Pack {
	final String _name;
	List<Pack> _packs;

	Project(this._name, {List<Pack> packs}) {
		_packs = packs;
	}

	@override
	String get name => _name;

	@override
	List<File> get files => [];

	@override
	onLoad(Context context) {
		return null;
	}

	@override
	onTick(Context context) {
		return null;
	}

	@override
	generate(Context context) {
		List<Widget> onLoads = [];
		List<Widget> onTicks = [];
		List<File> files = [];

		_packs.forEach((pack) {
			onLoads.add(File.execute('${pack.name}/load', child: pack.onLoad(context)));
			onTicks.add(File.execute('${pack.name}/tick', child: pack.onTick(context)));
			files.addAll(pack.files);
		});

		return Objd.Pack(
			name: _name,
			files: [...this.files, ...files],
			load: File('load', child: For.of([
				Detect.load(),
				Delay.load(),
				onLoad(context),
				...onLoads
			])),
			main: File('tick', child: For.of([
				onTick(context),
				...onTicks,
				Detect.tick(),
				Delay.tick()
			]))
		);
	}
}
