import 'package:objd/core.dart';

enum LogLevel {
	Silent,
	Warning,
	Info,
	Debug,
	Trace
}

class Logger {
	static LogLevel MinLogLevel = LogLevel.Silent;

	static Tellraw Log(LogLevel level, dynamic message) {
		List<TextComponent> parts = [];

		if (message is String) {
			parts.add(TextComponent(message));
		} else if (message is TextComponent) {
			parts.add(message);
		} else if (message is List<TextComponent>) {
			parts.addAll(message);
		} else if (message is List<dynamic>) {
			message.forEach((p) => parts.add(TextComponent(p.toString())));
		} else {
			parts.add(TextComponent(message.toString()));
		}

		return _canLog(level) ? Tellraw(Entity.All(), show: parts) : null;
	}

	static Tellraw Warning(dynamic message) {
		return Log(LogLevel.Warning, message);
	}

	static Tellraw Info(dynamic message) {
		return Log(LogLevel.Info, message);
	}

	static Tellraw Debug(dynamic message) {
		return Log(LogLevel.Debug, message);
	}

	static Tellraw Trace(dynamic message) {
		return Log(LogLevel.Trace, message);
	}

	static bool _canLog(LogLevel level) {
		switch(MinLogLevel) {
			case LogLevel.Trace: return [LogLevel.Trace].contains(level);
			case LogLevel.Debug: return [LogLevel.Debug, LogLevel.Trace].contains(level);
			case LogLevel.Info: return [LogLevel.Info, LogLevel.Debug, LogLevel.Trace].contains(level);
			case LogLevel.Warning: return [LogLevel.Warning, LogLevel.Info, LogLevel.Debug, LogLevel.Trace].contains(level);
			default: return false;
		}
	}
}
